/* mean.c */

#include "statistics.h"

#include <math.h>
#include <stdio.h>

double
compute_mean(unsigned nvalues, const double *values, unsigned tolerance,
	     double *sigma)
{
  unsigned i;
  double sum;
  double sum_squares;
  double mean;
  double dev;
  
  if (!nvalues)
    {
    empty:
      if (sigma)
	*sigma = 0.0;
      return 0.0;
    }

  for (i = 0, sum = sum_squares = 0.0; i < nvalues; i++)
    {
      double x = values[i];
      sum += x;
      sum_squares += x*x;
    }

  mean = sum / nvalues;
  if (nvalues < 2)
    dev = 0.0;
  else
    dev = sqrt( (sum_squares - sum*sum/nvalues)/(nvalues - 1) );

  if (tolerance && nvalues >= 2)
    {
      /* Recompute, discarding too large or two small values */
      double min = mean - tolerance * dev;
      double max = mean + tolerance * dev;
      unsigned n;
      
      for (n = i = 0, sum = sum_squares = 0.0; i < nvalues; i++)
	{
	  double x = values[i];
	  if (x > min && x < max)
	    {
	      sum += x;
	      sum_squares += x*x;
	      n++;
	    }
	}

      if (n < nvalues)
	{
	  /* Some values were skipped */

	  if (!n)
	    goto empty;
	  
	  mean = sum / nvalues;
	  if (n < 2)
	    dev = 0.0;
	  else
	    dev = sqrt( (sum_squares - sum*sum/n)/(n - 1) );
	}
    }

  if (sigma)
    *sigma = dev;
  return mean;
}
